package com.engine.qdhx.biz;

import com.engine.qdhx.entity.Context;
import com.engine.qdhx.entity.TaxTableEntity;
import com.engine.qdhx.entity.TaxTableNoResidentEntity;
import com.engine.qdhx.service.impl.AddSalaryStrategy;
import com.engine.qdhx.service.impl.MultiplySalaryStrategy;
import com.engine.qdhx.service.impl.SubtractionSalaryStrategy;
import com.engine.qdhx.util.SalaryUtil;
import com.engine.qdhx.util.TaxRateUtil;
import weaver.conn.RecordSet;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @Author weaver_cl
 * @Description: TODO 税率台账自动生成本月的个税
 * @Date 2022/3/2
 * @Version V1.0
 **/
public class HrmTaxDiffBiz {

    private final double MONEY = 5000; //非居民费用扣除标准

    private final double ZERO = 0;

    private final int JMFORMMODEID = 18;

    private final int FJMFORMMODEID = 28;

    //税率表获取(居民)
    private final List<TaxTableEntity> taxTable = TaxRateUtil.getTaxTable();
    //非居民
    private final List<TaxTableNoResidentEntity> taxTableNoResident = TaxRateUtil.getTaxTableNR();

    private DateTimeFormatter formatter1 = DateTimeFormatter.ofPattern("yyyyMM");


    /**
     * 计算居民个税
     * @param fromDate
     * @param workcode
     * @param shouldSendTotal
     * @return
     */
    public Map getTaxToDosByJm(String fromDate,String resourceid,String workcode,double shouldSendTotal,Map<String,Double> ipMap) {
        Map<String, List<Object>> datas = new HashMap<>();
        try {
            RecordSet rs = new RecordSet();
            Context context = new Context();
            context.setSalaryStrategy(new AddSalaryStrategy());
            Context context1 = new Context();
            context1.setSalaryStrategy(new SubtractionSalaryStrategy());
            Context context2 = new Context();
            context2.setSalaryStrategy(new MultiplySalaryStrategy());
            LocalDate start = LocalDate.parse(fromDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            Integer month = start.getMonthValue();

            //薪资所属年月
            String xzszny = formatter1.format(start);
            //个人所得税自定义项目
            Map<String,Double> customMap = TaxRateUtil.customTable(workcode, xzszny);

            //国家规定统一减免金额
            double gjgdtyjmje = customMap.getOrDefault("gjgdtyjmje",0.00);
            //个税起征点
            double gsqzd = taxTable.get(0).getQzd();
            //社保
            double sb = context.execute(ipMap.getOrDefault("ylgr16",0.00),ipMap.getOrDefault("sygr03",0.00),ipMap.getOrDefault("ylgr2",0.00));
            //公积金
            double gjj = ipMap.getOrDefault("gjjgr5",0.00);
            //专项附加扣除
            double zxfjkc = customMap.getOrDefault("zxfjkc",0.00);
            //依法确定的其他扣除
            double yfqddqtkc = customMap.getOrDefault("yfqddqtkc",0.00);

            //应纳税所得
            double ynssd = context1.execute(shouldSendTotal,gjgdtyjmje,gsqzd,sb,gjj,yfqddqtkc);

            rs.executeQuery("select sum(yfze) as ljsr, sum(gjgdtyjmje) as ljmssrgjtyjmje, sum(gsqzd) as ljjcfy," +
                    " sum(sb) as ljzxkcsb,sum(gjj) as ljzxkcgjj,sum(yfqddqtkc) as ljyfqddqtkc,sum(bqyykyjsedyyj) as ljyykyjse from uf_grsds where gh = ? and " +
                    " xzszny < ?",workcode,xzszny);

            rs.next();

            double ljsr = SalaryUtil.getDoubleValue(rs.getString("ljsr"),0.00);
            ljsr = context.execute(ljsr,shouldSendTotal); //累计收入

            //累计免税收入
            double ljmssrgjtyjmje = SalaryUtil.getDoubleValue(rs.getString("ljmssrgjtyjmje"),0.00);
            ljmssrgjtyjmje = context.execute(ljmssrgjtyjmje,gjgdtyjmje);
            //累加减除费用
            double ljjcfy = SalaryUtil.getDoubleValue(rs.getString("ljjcfy"),0.00);
            ljjcfy = context.execute(ljjcfy,gsqzd);
            //累计专项扣除（社保）
            double ljzxkcsb = SalaryUtil.getDoubleValue(rs.getString("ljzxkcsb"),0.00);
            ljzxkcsb = context.execute(ljzxkcsb,sb);
            //累计专项扣除（公积金）
            double ljzxkcgjj = SalaryUtil.getDoubleValue(rs.getString("ljzxkcgjj"),0.00);
            ljzxkcgjj = context.execute(ljzxkcgjj,gjj);
            //累计专项附加扣除
            double ljzxfjkc = context2.execute(zxfjkc,month);
            //累计依法确定的其他扣除
            double ljyfqddqtkc = SalaryUtil.getDoubleValue(rs.getString("ljyfqddqtkc"),0.00);
            ljyfqddqtkc = context.execute(ljyfqddqtkc,yfqddqtkc);
            //累计预扣预缴应纳税所得额
            double ljykyjynssde = context1.execute(ljsr,ljmssrgjtyjmje,ljjcfy,ljzxkcsb,ljzxkcgjj,ljzxfjkc,ljyfqddqtkc);

            //税率
            double sl = ZERO;
            //速算扣除数
            double sskcs = ZERO;
            //累计减免税额 (残疾人80%税)
            double ljjmsecjr80s = ZERO;
            //累计已预扣预缴税额
            double ljyykyjse = ZERO;
            //本期应预扣预缴税额
            double bqyykjse = ZERO;
            //本期应预扣预缴税额 (当月预计)
            double bqyykyjsedyyj = ZERO;
            if(ljykyjynssde <= 0) {
            }else {
                Integer side = TaxRateUtil.getTaxSide(ljsr,"0");
                TaxTableEntity taxTableEntity = taxTable.stream().filter(item -> item.getLjykyjynssde() == side).findAny().orElse(null);
                sl = taxTableEntity.getYsl();
                sskcs = taxTableEntity.getSskcs();
                ljyykyjse = SalaryUtil.getDoubleValue(rs.getString("ljyykyjse"),0.00);
                bqyykjse = context1.execute(context2.execute(ljykyjynssde,sl),sskcs,ljjmsecjr80s,ljyykyjse);
                bqyykyjsedyyj = bqyykjse;

            }


            List<Object> list = Arrays.asList(resourceid,workcode,xzszny,shouldSendTotal,
                    gjgdtyjmje,gsqzd,sb,gjj,zxfjkc,yfqddqtkc,ynssd,ljsr,ljmssrgjtyjmje,ljjcfy,
                    ljzxkcsb,ljzxkcgjj,ljzxfjkc,ljyfqddqtkc,ljykyjynssde,sl,sskcs,ljjmsecjr80s,ljyykyjse,bqyykjse,bqyykyjsedyyj,JMFORMMODEID);

            datas.put("taxRate",list);


        }catch (Exception e){
            e.printStackTrace();
        }
        return datas;

    }


    /**
     * 计算非居民个税
     * @param fromDate
     * @param workcode
     * @param shouldSendTotal
     * @return
     */
    public Map getTaxToDosByFjm(String fromDate,String resourceid,String workcode,double shouldSendTotal,Map<String,Double> ipMap) {
        Map<String, List<Object>> datas = new HashMap<>();
        try{
            Context context = new Context();
            context.setSalaryStrategy(new SubtractionSalaryStrategy());
            Context context1 = new Context();
            context1.setSalaryStrategy(new MultiplySalaryStrategy());
            LocalDate start = LocalDate.parse(fromDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            //薪资所属年月
            String xzszny = formatter1.format(start);
            //基本养老保险金
            double jbylbxj = ZERO;
            //公积金
            double zfgjj = ZERO;
            //其他免税所得
            double qtmssd = ZERO;
            //应税收入额
            double yssre = context.execute(shouldSendTotal,jbylbxj,zfgjj,qtmssd);
            double fykcbz = MONEY; //费用扣除标准
            double ynssde = context.execute(yssre,fykcbz); //应纳税所得额

            double sl = ZERO; //税率
            double sskcs = ZERO; //速算扣除数
            double ynse = ZERO; //应纳税额
            double jmse = ZERO; //减免税额
            double sjynse = ZERO; //实际应纳税额

            if (ynssde <= 0) {
            }else {
                Integer side = TaxRateUtil.getTaxSide(shouldSendTotal,"1");
                TaxTableNoResidentEntity taxTableNoResidentEntity = taxTableNoResident.stream().filter(item -> item.getYnssde() == side).findAny().orElse(null);
                sl = taxTableNoResidentEntity.getSl();
                sskcs = taxTableNoResidentEntity.getSskcs();
                ynse = context.execute(context1.execute(ynssde,sl),sskcs);
                sjynse = context.execute(ynse,jmse);
                //new BaseBean().writeLog(String.format("side: %s || taxTableNoResident %s",side,taxTableNoResident.toString()));
            }


            double shyx = context.execute(shouldSendTotal,jbylbxj,zfgjj,sjynse); //税后月薪

            List<Object> list = Arrays.asList(resourceid,workcode,xzszny,shouldSendTotal,jbylbxj,zfgjj,qtmssd,yssre,
                    fykcbz,ynssde,sl,sskcs,ynse,jmse,sjynse,shyx,FJMFORMMODEID);

            datas.put("taxRateNR",list);


        }catch (Exception e) {
            e.printStackTrace();
        }


        return datas;
    }
}
