package com.engine.qdhx.util;

import com.engine.kq.biz.KQLeaveRulesComInfo;
import com.engine.kq.biz.KQOvertimeRulesBiz;
import com.engine.kq.enums.KqSplitFlowTypeEnum;
import com.engine.kq.util.KQDurationCalculatorUtil;
import com.engine.qdhx.entity.Context;
import com.engine.qdhx.service.impl.AddSalaryStrategy;
import com.engine.qdhx.service.impl.DivisionSalaryStrategy;
import com.engine.qdhx.service.impl.MultiplySalaryStrategy;
import com.engine.qdhx.service.impl.SubtractionSalaryStrategy;
import lombok.SneakyThrows;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.Util;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;


/**
 * @Author weaver_cl
 * @Description: TODO
 * @Date 2022/2/21
 * @Version V1.0
 **/
public class SalaryUtil extends BaseBean {

    private static final String SCOPE = "HrmCustomFieldByInfoType";

    private static final Integer SCOPEID = -1;

    private static final Integer SCOPEID2 = 1;

    private static final Integer TAXFIELDID = 4;

    private static final double AVERAGE = 21.75;

    private static DecimalFormat df = new DecimalFormat("0.00");



    /**
     * 获取处名称
     * @param id
     * @return
     */
    public static String getPlaceById(String id) {
        RecordSet rs = new RecordSet();
        rs.executeQuery("select field1 from cus_fielddata where " +
                "scope = ? and scopeid = ? and id = ?",SCOPE,SCOPEID,id);

        return rs.next()?Util.null2String(rs.getString("field1")):"";

    }

    /**
     * 获取费用代码
     * @param id
     * @return
     */
    public static String getCostcodeById(String id) {
        RecordSet rs = new RecordSet();
        rs.executeQuery("select field3 from cus_fielddata where " +
                "scope = ? and scopeid = ? and id = ?",SCOPE,SCOPEID,id);

        return rs.next()?Util.null2String(rs.getString("field3")):"";

    }

    /**
     * 获取税务身份
     * @param id
     * @return
     */
    public static String getTaxidById(String id) {
        RecordSet rs = new RecordSet();
        rs.executeQuery("select field4 from cus_fielddata where " +
                "scope = ? and scopeid = ? and id = ?",SCOPE,SCOPEID2,id);

        return rs.next()?rs.getString("field4"):"";
    }


    /**
     * 获取税务身份真实值
     * @param selectvalue
     * @return
     */
    public static String getTaxName(String selectvalue) {
        RecordSet rs = new RecordSet();
        rs.executeQuery("select selectname from cus_selectitem where fieldid = ?" +
                " and selectvalue = ?",TAXFIELDID,selectvalue);
        String selectname = "";
        if (rs.next()) {
            selectname = Util.null2String(rs.getString("selectname"));
        }
        return selectname;
    }


    /**
     * 获取基本工资
     * @param id
     * @return
     */
    public static Double getBasicSalary(String id) {
        RecordSet rs = new RecordSet();
        rs.executeQuery("select field6 from cus_fielddata where " +
                "scope = ? and scopeid = ? and id = ?",SCOPE,SCOPEID2,id);
        rs.next();
        return getDoubleValue(rs.getString("field6"),0.00);
    }

    /**
     * 获取固定加班费
     * @param id
     * @return
     */
    public static Double getOvertimePay(String id) {
        RecordSet rs = new RecordSet();
        rs.executeQuery("select field5 from cus_fielddata where " +
                "scope = ? and scopeid = ? and id = ?",SCOPE,SCOPEID2,id);
        rs.next();
        return getDoubleValue(rs.getString("field5"),0.00);
    }

    /**
     * 获取中国农业银行账户
     * @param id
     * @return
     */
    public static String getBankAccount(String id) {
        RecordSet rs = new RecordSet();
        rs.executeQuery("select field29 from cus_fielddata where " +
                "scope = ? and scopeid = ? and id = ?",SCOPE,SCOPEID2,id);
        String bankAccount = "";
        if(rs.next()) {
            bankAccount = Util.null2String(rs.getString("field29"));
        }
        return bankAccount;
    }

    /**
     * 判断日期存在日期区间
     * @param startDate
     * @param endDate
     * @param date
     * @return
     */
    public static boolean dateInterval(String startDate,String endDate,String date){
        try {
            if (date.length()<=0){
                return false;
            }
            LocalDate start = LocalDate.parse(startDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            LocalDate end = LocalDate.parse(endDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            LocalDate now = LocalDate.parse(date, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            return now.isAfter(start) && now.isBefore(end);
        }catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }



    /**
     * 获取建模(uf_flxm)福利项目数据
     * @param workcode
     * @return
     */
    public static Map welfareTable(String workcode){
        RecordSet cube = new RecordSet();
        Map<String,Double> welfare = new HashMap<>();
        cube.executeQuery("select birthdayCashgift,merryCashgift,funeralCashgift,festivalCashgift,preventHeatAllowance," +
                " highTemperatureSubsidy,communicatySubsidy,shiftAllowance,standAllowance," +
                " achievementonus,abroadInsuranceSubsidy,otherItem1,otherItem2,houseSubsidy from uf_flxm where workcode = ?",workcode);

        List<String> welfareList = Stream.of("birthdayCashgift","merryCashgift","funeralCashgift","festivalCashgift",
                "preventHeatAllowance","highTemperatureSubsidy","communicatySubsidy","shiftAllowance",
                "standAllowance","achievementonus","abroadInsuranceSubsidy","otherItem1","otherItem2","houseSubsidy").collect(Collectors.toList());
        if (cube.next()){
            welfareList.forEach(field -> {
                welfare.put(field,getDoubleValue(cube.getString(field),0.00));
            });
        }

        return welfare;
    }


    /**
     * 高温和防暑补贴
     * @param day
     * @param money
     * @return
     */
    public static double heatAndTemperatureAllowance (double day,double money){
        Context context = new Context();
        context.setSalaryStrategy(new DivisionSalaryStrategy());
        money = context.execute(money,AVERAGE);
        context.setSalaryStrategy( new MultiplySalaryStrategy());
        double value = context.execute(money,day);
        return value;
    }

    /**
     * 获取建模(uf_jj)奖金数据
     * @param workcode
     * @return
     */
    public static Map bonusTable(String workcode){
        RecordSet cube = new RecordSet();
        Map<String,Object> bonus = new HashMap<>();
        cube.executeQuery("select specialAward,signMoney,meritsPay,annualBonus,ompensation from uf_jj where workcode = ?",workcode);
        if (cube.next()){
            List<String> bonusList = Stream.of("specialAward","signMoney","meritsPay","annualBonus","ompensation").collect(Collectors.toList());
            bonusList.forEach(field -> {
                bonus.put(field,getDoubleValue(cube.getString(field),0.00));
            });
        }

        return bonus;
    }

    /**
     * 获取建模(uf_gzjx)扣款数据
     * @param workcode
     * @return
     */
    public static Map reduceTable(String workcode){
        RecordSet cube = new RecordSet();
        Map<String,Object> reduce = new HashMap<>();
        cube.executeQuery("select otherDeduction,costWithhold,mealWithhold from uf_gzjx where workcode = ?",workcode);
        if(cube.next()) {
            List<String> reduceList = Stream.of("otherDeduction","costWithhold","mealWithhold").collect(Collectors.toList());
            reduceList.forEach(field -> {
                reduce.put(field,getDoubleValue(cube.getString(field),0.00));
            });
        }

        return reduce;
    }

    /**
     * 判断端午中秋是否在指定日期
     * @param startDate
     * @param endDate
     * @return
     */
    public static boolean judgeHoliday(String startDate,String endDate) {
        RecordSet rs = new RecordSet();
        boolean result = false;
            List<String> linkList = new LinkedList<>();
            rs.executeQuery("select MIN(holidayDate) as holidaydate,holidayDesc from KQ_HolidaySet where holidaydesc in ('端午节','中秋节') group by holidaydesc");
            while (rs.next()) {
                String holidaydate = Util.null2String(rs.getString("holidaydate"));
                linkList.add(holidaydate);
            }
            for(String date : linkList){
                result = dateInterval(startDate,endDate,date);
                if (result) break;
            }
            return result;
    }

    /**
     * 加班金额计算 G1(工作日)=加班时数*人民币基本工资/21.75/8*1.5
     * @param basicSalary
     * @param hour
     * @param type
     * @return
     */
    public static double overTimeWorkMoney(double basicSalary, String hour,String type) {
        BigDecimal salary = new BigDecimal(String.valueOf(basicSalary));
        BigDecimal day = new BigDecimal("21.75");
        BigDecimal hours = new BigDecimal("8");
        BigDecimal hourBig = new BigDecimal(hour);
        BigDecimal times = new BigDecimal(type);
        return (hourBig.multiply(salary.divide(day,20,BigDecimal.ROUND_HALF_UP)
                .divide(hours,20,BigDecimal.ROUND_HALF_UP).multiply(times))).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }


    /**
     * 获取考勤信息
     * @param startDate
     * @param endDate
     * @param resourceid
     * @return
     */
    public static Map getKqInfo(String startDate,String endDate,String resourceid){
        RecordSet rs = new RecordSet();
        Map<String,Double> kqMap = new HashMap<>();
        rs.executeQuery("select sum(workdays) as workdays, sum(attendancedays) as attendancedays," +
                " sum(absenteeismmins) as absenteeismmins," +
                " sum(belatemins) as belatemins,sum(leaveearlymins) as leaveearlymins from kq_format_total where " +
                " resourceid = ? and kqdate >= ? and kqdate <= ?",resourceid,startDate,endDate);
        if (rs.next()){
            double workdays = getDoubleValue(rs.getString("workdays"),0);
            double attendancedays = getDoubleValue(rs.getString("attendancedays"),0);
            //旷工时长 （分钟）
            double absenteeismmins = getDoubleValue(rs.getString("absenteeismmins"),0);
            //迟到时长（分钟）
            double belatemins = getDoubleValue(rs.getString("belatemins"),0);
            //早退时长 (分钟）
            double leaveearlymins = getDoubleValue(rs.getString("leaveearlymins"),0);

            Context context = new Context();
            //context.setSalaryStrategy(new SubtractionSalaryStrategy());
            //double mins = context.execute(absenteeismmins,belatemins,leaveearlymins);
            context.setSalaryStrategy(new DivisionSalaryStrategy());

            //请假未归档天数(不扣款) --换算成天
            double leavedays = context.execute(absenteeismmins,480);

            kqMap.put("workdays",workdays);
            kqMap.put("attendancedays",attendancedays);
            kqMap.put("leavedays",leavedays);
        }


        return kqMap;
    }


    /**
     * 考勤扣款(缺勤天数*日薪)
     * @param money
     * @param workdays
     * @param attendancedays
     * @return
     */
    public static double attendanceDeduction(double money,double workdays,double attendancedays) {
        if (new BigDecimal(workdays).equals(BigDecimal.ZERO)){
            return 0;
        }
        Context context = new Context();
        context.setSalaryStrategy(new SubtractionSalaryStrategy());
        double day = context.execute(workdays,attendancedays);
        day = day>0?day:0;
        context.setSalaryStrategy(new DivisionSalaryStrategy());
        money = context.execute(money,workdays);
        context.setSalaryStrategy(new MultiplySalaryStrategy());
        money = context.execute(money,day);
        return money;
    }

    /**
     * 获取加班时长（工作日，双休，节假日） (薪资月)
     * @param fromDate
     * @param toDate
     * @param params
     * @return
     */
    public static Map getFlowOverTimeDataNew(String fromDate,String toDate,Map<String,String> params) {
        Map<String,Object> overTime = new HashMap<>();
        BaseBean bb = new BaseBean();
        RecordSet rst = new RecordSet();
        try {
            KQOvertimeRulesBiz kqOvertimeRulesBiz = new KQOvertimeRulesBiz();

            String sqlWhere = " ";
            if(params.get("subcompanyid1").length()>0){
                sqlWhere +=" and a.subcompanyid1 in("+params.get("subcompanyid1")+") ";
            }

            if(params.get("departmentid").length()>0){
                sqlWhere +=" and a.departmentid in("+params.get("departmentid")+") ";
            }

            if(params.get("id").length()>0){
                sqlWhere +=" and a.id in("+params.get("id")+") ";
            }

            int uintType = KQOvertimeRulesBiz.getMinimumUnit();//当前加班单位
            double hoursToDay = KQOvertimeRulesBiz.getHoursToDay();//当前天跟小时计算关系

            String sql = " select resourceid,changeType, sum(cast(duration_min as decimal(18,4))) as val,paidLeaveEnable "+
                    " from hrmresource a, kq_flow_overtime b "+
                    " where a.id = b.resourceid and belongdate >='"+fromDate+"' and belongdate <='"+toDate+"' " +sqlWhere+
                    " group by resourceid,changeType,paidLeaveEnable";
            rst.executeQuery(sql);
            while (rst.next()) {
                String resourceid = rst.getString("resourceid");
                String paidLeaveEnable = rst.getString("paidLeaveEnable");
                int changeType =rst.getInt("changeType");//1-节假日、2-工作日、3-休息日
                double value = rst.getDouble("val")<0?0:rst.getDouble("val");
                if(uintType==3 || uintType== 5 || uintType== 6){//按小时计算
                    value = Util.getDoubleValue(KQDurationCalculatorUtil.getDurationRound(value/(60.0)+""));
                }else{//按天计算
                    value = Util.getDoubleValue(KQDurationCalculatorUtil.getDurationRound(value/(60.0*hoursToDay)+""));
                }
                String flowType = "";
                if(changeType==1){
                    flowType = "holidayOvertime";
                }else if(changeType==2){
                    flowType = "workingDayOvertime";
                }else if(changeType==3){
                    flowType = "restDayOvertime";
                }
                if("1".equalsIgnoreCase(paidLeaveEnable)){
                    //1表示关联调休
                    flowType += "_4leave";
                }else{
                    //0表示不关联调休
                    flowType += "_nonleave";
                }
                String key = resourceid+"|"+flowType;
                //df.format 默认是不四舍五入的 0.125这样的就会直接变成0.12了
                df.setMaximumFractionDigits(5);
                if(overTime.containsKey(key)){
                    double tmpVal = Util.getDoubleValue(Util.null2String(overTime.get(key)),0.0);
                    tmpVal += value;
                    overTime.put(key,df.format(tmpVal));
                }else{
                    overTime.put(key,df.format(value));
                }

            }

        }catch (Exception e) {
            e.printStackTrace();
        }


        return overTime;
    }

    /**
     *  获取加班时长（工作日，双休，节假日） (薪资上月未结算的加班)
     * @param fromDate
     * @param toDate
     * @param params
     * @return
     */
    public static Map getLastFlowOverTimeData(String fromDate,String toDate,Map<String,String> params) {
        Map<String,Object> overTime = new HashMap<>();
        RecordSet rst = new RecordSet();
        try {
            KQOvertimeRulesBiz kqOvertimeRulesBiz = new KQOvertimeRulesBiz();
            fromDate = getSameDay(fromDate);
            toDate = getSameDay(toDate);
            DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            LocalDate start = LocalDate.parse(fromDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            //薪资结算日期
            String settlementDate = LocalDate.of(start.getYear(), start.getMonth(), 6).format(fmt);

            String sqlWhere = " ";
            if(params.get("subcompanyid1").length()>0){
                sqlWhere +=" and a.subcompanyid1 in("+params.get("subcompanyid1")+") ";
            }

            if(params.get("departmentid").length()>0){
                sqlWhere +=" and a.departmentid in("+params.get("departmentid")+") ";
            }

            if(params.get("id").length()>0){
                sqlWhere +=" and a.id in("+params.get("id")+") ";
            }

            int uintType = kqOvertimeRulesBiz.getMinimumUnit();//当前加班单位
            double hoursToDay = kqOvertimeRulesBiz.getHoursToDay();//当前天跟小时计算关系

            String sql = "SELECT b.requestid,b.resourceid,b.changeType,sum(cast(b.duration_min AS DECIMAL ( 18, 4 ))) AS val,b.paidLeaveEnable " +
                    " FROM kq_flow_overtime b " +
                    " INNER JOIN hrmresource a ON a.id = b.resourceid and b.belongdate >='"+fromDate+"' and b.belongdate <='"+toDate+"' " +sqlWhere+
                    " LEFT JOIN workflow_requestbase c on b.requestid = c.requestid and c.currentnodetype = 3" +
                    " and c.lastoperatedate >= '"+settlementDate+"'" +
                    " group by resourceid,changeType,paidLeaveEnable,b.requestid";
            rst.executeQuery(sql);
            while (rst.next()) {
                String resourceid = rst.getString("resourceid");
                String paidLeaveEnable = rst.getString("paidLeaveEnable");
                int changeType =rst.getInt("changeType");//1-节假日、2-工作日、3-休息日
                double value = rst.getDouble("val")<0?0:rst.getDouble("val");
                if(uintType==3 || uintType== 5 || uintType== 6){//按小时计算
                    value = Util.getDoubleValue(KQDurationCalculatorUtil.getDurationRound(value/(60.0)+""));
                }else{//按天计算
                    value = Util.getDoubleValue(KQDurationCalculatorUtil.getDurationRound(value/(60.0*hoursToDay)+""));
                }
                String flowType = "";
                if(changeType==1){
                    flowType = "holidayOvertime";
                }else if(changeType==2){
                    flowType = "workingDayOvertime";
                }else if(changeType==3){
                    flowType = "restDayOvertime";
                }
                if("1".equalsIgnoreCase(paidLeaveEnable)){
                    //1表示关联调休
                    flowType += "_4leave";
                }else{
                    //0表示不关联调休
                    flowType += "_nonleave";
                }
                String key = resourceid+"|"+flowType;
                //df.format 默认是不四舍五入的 0.125这样的就会直接变成0.12了
                df.setMaximumFractionDigits(5);
                if(overTime.containsKey(key)){
                    double tmpVal = Util.getDoubleValue(Util.null2String(overTime.get(key)),0.0);
                    tmpVal += value;
                    overTime.put(key,df.format(tmpVal));
                }else{
                    overTime.put(key,df.format(value));
                }

            }

        }catch (Exception e) {
            e.printStackTrace();
        }


        return overTime;
    }


    /**
     * 获取请假数据
     * @param params
     * @return
     */
    public static Map<String,String> getFlowLeaveData(String fromDate,String toDate,Map<String,String> params){
        Map<String,String> datas = new HashMap<>();
        RecordSet rs = new RecordSet();
        String sql = "";
        String sqlWhere = " ";
        try{
            KQLeaveRulesComInfo kqLeaveRulesComInfo = new KQLeaveRulesComInfo();
            if(params.get("subcompanyid1").length()>0){
                sqlWhere +=" and a.subcompanyid1 in("+params.get("subcompanyid1")+") ";
            }

            if(params.get("departmentid").length()>0){
                sqlWhere +=" and a.departmentid in("+params.get("departmentid")+") ";
            }

            if(params.get("id").length()>0){
                sqlWhere +=" and a.id in("+params.get("id")+") ";
            }
            sql = " select resourceid, newleavetype, durationrule, sum(duration) as val from hrmresource a, "+ KqSplitFlowTypeEnum.LEAVE.getTablename()+" b "+
                    " where a.id = b.resourceid and belongdate >='"+fromDate+"' and belongdate <='"+toDate+"' " +sqlWhere	+
                    " group by resourceid, newleavetype, durationrule ";
            rs.execute(sql);
            while (rs.next()) {
                String resourceid = rs.getString("resourceid");
                String newleavetype = rs.getString("newleavetype");
                String durationrule = rs.getString("durationrule");
                double value = rs.getDouble("val")<0?0:rs.getDouble("val");

                double proportion = Util.getDoubleValue(kqLeaveRulesComInfo.getProportion(newleavetype));
//                if(isLeaveHour(newleavetype, kqLeaveRulesComInfo)){//按小时
//                    if(!isLeaveHour(durationrule)){
//                        if(proportion>0) value = value*proportion;
//                    }
//                }else{//按天
//                    if(isLeaveHour(durationrule)){
//                        if(proportion>0) value = value/proportion;
//                    }
//                }

                if (isLeaveHour(newleavetype,kqLeaveRulesComInfo)) { //按小时
                    if (proportion>0) value = value/proportion;
                }

                String key = resourceid+"|leaveType_"+newleavetype;
                if(datas.containsKey(key)){
                    value += Util.getDoubleValue(Util.null2String(datas.get(key)));
                }
                //df.format 默认是不四舍五入的 0.125这样的就会直接变成0.12了
                df.setMaximumFractionDigits(5);
                datas.put(key,df.format(value));
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return datas;
    }

    /**
     * 请假数据换算工资
     * @param leaveData
     * @param resourceid
     * @return
     */
    public static double conversionLeave(Map<String,String> leaveData,String resourceid) {
        double result = 0.00;
        BaseBean bb = new BaseBean();
        //bb.writeLog(String.format("leave data %s",leaveData.toString()));
        Context context = new Context();
        context.setSalaryStrategy(new AddSalaryStrategy());
        Context context1 = new Context();
        context.setSalaryStrategy(new MultiplySalaryStrategy());
        //leaveData.get(resourceid + "|leaveType_2");//事假无薪资
        //病假 70%薪资 医疗期 60% 薪资 剩余假期类型 全薪资
        result = context.execute(result,context1.execute(getDoubleValue(leaveData.get(resourceid + "|leaveType_3"),0),0.7),
                context1.execute(getDoubleValue(leaveData.get(resourceid + "|leaveType_11"),0),0.6),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_4"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_5"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_6"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_7"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_8"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_9"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_10"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_12"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_13"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_14"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_15"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_16"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_17"),0),
                getDoubleValue(leaveData.get(resourceid + "|leaveType_18"),0));

        return result;
    }

    /**
     * 根据单位类型 判断请假是否是小时为单位
     * @param durationrule
     * @return
     */
    public static boolean isLeaveHour(String durationrule){
        if((durationrule.equals("2") || durationrule.equals("3") || durationrule.equals("9")
                || durationrule.equals("11") || durationrule.equals("12") || durationrule.equals("14")
                || durationrule.equals("16"))){
            return true;
        }
        return false;
    }

    /**
     * 根据请假类型 判断请假是否是小时为单位
     * @param newleavetype
     * @param kqLeaveRulesComInfo
     * @return
     */
    public static boolean isLeaveHour(String newleavetype,KQLeaveRulesComInfo kqLeaveRulesComInfo){
        String durationrule = kqLeaveRulesComInfo.getMinimumUnit(newleavetype);
        return isLeaveHour(durationrule);
    }





    /**
     * 获取double值
     * @param v
     * @param def
     * @return
     */
    public static double getDoubleValue(String v, double def) {
        try {
            if (v != null && v.length()>0){
                return Double.parseDouble(v);
            }else {
                return def;
            }
        } catch (Exception e) {
            return def;
        }
    }


    /**
     * 获取社保公积金
     * @param workcode
     * @param xzszny
     * @return
     */
    public static Map getInsuranceAndProvident(String workcode,String xzszny){
        RecordSet cube = new RecordSet();
        Map<String,Object> insurance = new HashMap<>();
        cube.executeQuery("select ylgr16,sygr03,ylgr2,gjjgr5 from uf_wxyjftb where gh = ? and ny = ?",workcode,xzszny);
        if (cube.next()) {
            List<String> reduceList = Stream.of("ylgr16","sygr03","ylgr2","gjjgr5").collect(Collectors.toList());
            reduceList.forEach(field -> {
                insurance.put(field,getDoubleValue(cube.getString(field),0.00));
            });
        }

        return insurance;
    }



    /**
     * 比较第一个日期是否大于第二个日期
     * @param firstDate 第一个日期
     * @param secondDate 第二个日期
     * @return true-大于;false-不大于
     */
    public static boolean localDateIsBefore(String firstDate, String secondDate) {
        if (secondDate.length()<=0 || "".equals(secondDate)){
            return false;
        }
        LocalDate first = LocalDate.parse(firstDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        LocalDate second = LocalDate.parse(secondDate, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        return first.isBefore(second);
    }


    /**
     * 获取离职日期(作废)
     * @param id
     * @return
     */
    public static String getDismissDate(String id) {
        RecordSet rs = new RecordSet();
        rs.executeQuery("select dismissdate from bill_hrmdismiss where resource_n = ?",id);
        rs.next();
        return Util.null2String(rs.getString("dismissdate"));
    }

    /**
     * 获取上个月同一天
     * @param date
     * @return
     */
    @SneakyThrows
    public static String getSameDay(String date) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        Date lastDate = dateFormat.parse(date);
        calendar.setTime(lastDate); // 设置为当前时间
        calendar.set(Calendar.MONTH, calendar.get(Calendar.MONTH) - 1); // 设置为上一个月
        return dateFormat.format(calendar.getTime());
    }

}
