package com.engine.qdhx.util;

import com.engine.qdhx.entity.TaxTableEntity;
import com.engine.qdhx.entity.TaxTableNoResidentEntity;
import weaver.conn.RecordSet;
import weaver.general.Util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Author weaver_cl
 * @Description: TODO
 * @Date 2022/3/2
 * @Version V1.0
 **/
public class TaxRateUtil {

    //税率范围下拉框(居民)
    private static final String FIELDID = "8190";
    //税率范围下拉框(非居民)
    private static final String FIELDID1 = "8268";



    /**
     * 税率表获取（居民）
     * @return
     */
    public static List<TaxTableEntity> getTaxTable(){
        RecordSet rs = new RecordSet();
        List<TaxTableEntity> list = new ArrayList<>();
        rs.executeQuery("select qzd,ljykyjynssde,ysl,sskcs,sxzt,bz from uf_jmslb where sxzt = 0");
        while (rs.next()) {
            double qzd = SalaryUtil.getDoubleValue(rs.getString("qzd"),0.00);
            int ljykyjynssde = Util.getIntValue(rs.getString("ljykyjynssde"));
            double ysl = Util.getDoubleValue(rs.getString("ysl"));
            double sskcs = SalaryUtil.getDoubleValue(rs.getString("sskcs"),0.00);
            int sxzt = Util.getIntValue(rs.getString("sxzt"));
            String bz = Util.null2String(rs.getString("bz"));
            TaxTableEntity taxTableEntity = new TaxTableEntity(qzd,ljykyjynssde,ysl,sskcs,sxzt,bz);
            list.add(taxTableEntity);
        }
        return list;
    }

    /**
     * 税率表获取（非居民)
     * @return
     */
    public static List<TaxTableNoResidentEntity> getTaxTableNR(){
        RecordSet rs = new RecordSet();
        List<TaxTableNoResidentEntity> list = new ArrayList<>();
        rs.executeQuery("select ynssde,sl,sskcs from uf_fjmslb");
        while (rs.next()) {
            int ynssde = Util.getIntValue(rs.getString("ynssde"));
            double sl = Util.getDoubleValue(rs.getString("sl"));
            double sskcs = SalaryUtil.getDoubleValue(rs.getString("sskcs"),0.00);
            TaxTableNoResidentEntity taxTableNoResidentEntity = new TaxTableNoResidentEntity(ynssde,sl,sskcs);
            list.add(taxTableNoResidentEntity);
        }
        return list;
    }




    /**
     * 个人所得税自定义项目
     * @param workcode
     * @param xzszny
     * @return
     */
    public static Map customTable(String workcode, String xzszny){
        RecordSet rs = new RecordSet();
        Map<String,Object> custom = new HashMap<>();
        rs.executeQuery("select gjgdtyjmje,zxfjkc,yfqddqtkc from uf_grsdszdy where gh = ? and szny = ?",workcode,xzszny);
        rs.next();
        List<String> bonusList = Stream.of("gjgdtyjmje","zxfjkc","yfqddqtkc","annualBonus","ompensation").collect(Collectors.toList());
        bonusList.forEach(field -> {
            custom.put(field,SalaryUtil.getDoubleValue(rs.getString(field),0.00));
        });

        return custom;
    }

    /**
     * 获取税率表范围
     * @param ljsr
     * @return
     */
    public static int getTaxSide(double ljsr,String taxId){
        RecordSet rs = new RecordSet();
        String fieldId = "0".equals(taxId) ? FIELDID : FIELDID1;
        int side = 0;
        rs.executeQuery("select selectvalue,selectname from workflow_SelectItem where fieldid = ?",fieldId);
        while (rs.next()) {
            int selectvalue = Util.getIntValue(rs.getString("selectvalue"));
            String selectname = Util.null2String(rs.getString("selectname"));
            if (selectname.contains("-")){
                double min = Double.valueOf(selectname.split("-")[0]);
                double max = Double.valueOf(selectname.split("-")[1]);
                if (ljsr>min && ljsr<max){
                    side = selectvalue;
                    break;
                }
            }else {
                double value = Double.valueOf(selectname);
                if (ljsr > value){
                    side = selectvalue;
                    break;
                }
            }
        }
        return side;
    }
}
